#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  9/14/2021 3:36 PM
@Desc    :  Hook line.
"""
import logging
import random

import allure
import pytest

from logging import getLogger
from airtest.core.api import *
from airtest.core.error import AdbError
from airtest.core.helper import device_platform, ST
from popups.dismiss import popup, UT
from selenium.webdriver.chrome.options import Options
from selenium import webdriver
from webdriver_manager.chrome import ChromeDriverManager
from tenacity import Retrying, wait_fixed, stop_after_attempt
from wda import WDAError
from utx.core.css.custom_report import gen_report
from utx.core.utils.tools import display
from utx.core.utils.tools import str_to_bool, check_port, decryption, plat, download, device_udid, cleanup, proxy, \
    check_network

os.environ['WDM_SSL_VERIFY'] = '0'
logger = getLogger(__name__)

ST.THRESHOLD = 0.7
ST.OPDELAY = 0.25
ST.FIND_TIMEOUT = 10
ST.FIND_TIMEOUT_TMP = 2
ST.SNAPSHOT_QUALITY = 10

# todo https://github.com/openutx/utx-pypi/issues/4
# 1.全部都截图，不管成功失败
# 2.只有失败才截图
# 3.配置化
ST.SAVE_IMAGE = True


def app_info(cli_platform, cli_device, cli_wda, cli_init, ini_platform, ini_device, ini_wda, ini_init):
    """

    :param cli_platform:
    :param cli_device:
    :param cli_wda:
    :param cli_init:
    :param ini_platform:
    :param ini_device:
    :param ini_wda:
    :param ini_init:
    :return: iOS:///127.0.0.1:8100
    """
    if cli_init in [None, '']:
        is_init = str_to_bool(ini_init)
    else:
        is_init = str_to_bool(cli_init)
    if cli_wda in [None, '']:
        web_driver_agent = ini_wda
        # web_driver_agent = "*WebDriverAgent*"
    else:
        web_driver_agent = cli_wda
        # web_driver_agent = "*WebDriverAgent*"
    # 改为通配符，避免传入错误 bundle_id 启动失败

    if cli_platform in [None, '']:
        platform = ini_platform.lower()
        if platform in 'android':
            device_idx = str(ini_device).split(',')
            device_uri = []
            for idx in device_idx:
                device_uri.append(
                    'Android:///{}?cap_method=MINICAP&&ori_method=MINICAPORI&&touch_method=MAXTOUCH'.format(idx))
            return device_uri, device_idx, is_init
        elif platform in 'ios':
            device_idx = []
            device_uri = []
            for u in device_udid(types='ios', state=None):
                proxy_port = str(random.randint(50000, 60000) + 1)
                check_port(port=proxy_port)
                os.system('{} -u {} wdaproxy -B {} --port {} &'.format(decryption(b'dGlkZXZpY2U='), u, web_driver_agent,
                                                                       proxy_port))
                device_idx.append('127.0.0.1:{}'.format(proxy_port))
                device_uri.append('iOS:///127.0.0.1:{}'.format(proxy_port))
            return device_uri, device_idx, is_init

    elif cli_platform not in [None, '']:
        if cli_platform.lower() in 'android':
            device_idx = str(cli_device).split(',')
            device_uri = []
            for idx in device_idx:
                device_uri.append(
                    'Android:///{}?cap_method=MINICAP&&ori_method=MINICAPORI&&touch_method=MAXTOUCH'.format(idx))
            return device_uri, device_idx, is_init
        elif cli_platform.lower() in 'ios':
            device_idx = []
            device_uri = []
            for u in device_udid(types='ios', state=None):
                proxy_port = str(random.randint(50000, 60000) + 1)
                check_port(port=proxy_port)
                os.system(
                    '{} -u {} wdaproxy -B {} --port {} &'.format(decryption(b'dGlkZXZpY2U='), u, web_driver_agent,
                                                                 proxy_port))
                device_idx.append('127.0.0.1:{}'.format(proxy))
                device_uri.append('iOS:///127.0.0.1:{}'.format(proxy_port))
            return device_uri, device_idx, is_init


def my_before_sleep(retry_state):
    """

    :param retry_state:
    :return:
    """
    if retry_state.attempt_number < 1:
        loglevel = logging.INFO
    else:
        loglevel = logging.WARNING

    logging.log(
        loglevel,
        'Retrying %s: attempt %s ended with: %s',
        retry_state.fn,
        retry_state.attempt_number,
        retry_state.outcome,
    )


@allure.step("Try to link the device！")
def my_retry_connect(uri=None, whether_retry=True, sleeps=10, max_attempts=3):
    """

    :param uri:
    :param whether_retry:
    :param sleeps:
    :param max_attempts:
    :return:
    """
    if not whether_retry:
        max_attempts = 1

    r = Retrying(wait=wait_fixed(sleeps), stop=stop_after_attempt(max_attempts), before_sleep=my_before_sleep,
                 reraise=True)
    try:
        return r(connect_device, uri)
    except Exception as e:
        if isinstance(e, (WDAError,)):
            logger.info("Can't connect iphone, please check device or wda state!")
        logger.info("Try connect device {} 3 times per wait 10 sec failed.".format(uri))
        raise e
    finally:
        logger.info("Retry connect statistics: {}".format(str(r.statistics)))


@allure.step("Switch to current device！")
def ensure_current_device(device_idx):
    """

    :param device_idx:
    :return:
    """
    idx = device_idx
    try:
        if device().uuid != idx:
            set_current(idx)
    except IndexError:
        if device().uuid != 'http://{}'.format(idx):
            set_current(idx)


@allure.step("Try to wake up the current device！")
def wake_device(current_device):
    """

    :param current_device:
    :return:
    """
    try:
        current_device.wake()
        if current_device.is_locked():
            w, h = display()
            swipe((0.5 * w, 0.8 * h), (0.5 * w, 0.2 * h), duration=0.1)
        current_device.home()
    except AttributeError:
        pass


def app_fixture(request, device_uri, app_filepath, app_name, device_idx, init, agent=False):
    """

    :param request: default param
    :param device_uri: device_uri
    :param app_filepath: app_filepath
    :param app_name: app_name
    :param device_idx: device udid
    :param init: initial install or uninstall
    :param agent: Proxy status is off by default
    :return: app
    """
    with allure.step("Initialize and generate APP object！"):
        logger.info("Session start test.")

        try:
            app = None
            if init:
                logger.info("Detected that the initialization is True and started installing the application！")
                if str(device_uri[0]).startswith('iOS'):
                    if 'http' in app_filepath:
                        app_pkg = download(file_url=app_filepath, types='ipa')
                    else:
                        app_pkg = app_filepath
                else:
                    if 'http' in app_filepath:
                        app_pkg = download(file_url=app_filepath, types='apk')
                    else:
                        app_pkg = app_filepath
            for (uri, idx) in zip(device_uri, device_idx):
                app = my_retry_connect(uri)
                wake_device(G.DEVICE)
                if device_platform().lower() in 'android':
                    if not check_network():
                        warning_report = os.path.join(app_filepath.split('packages')[0], 'report', 'airtest')
                        gen_report(results=[{'result': []}], report_path=warning_report)
                        # todo: xfail is skip ,fail is fail. Lack of multi-device differentiation.
                        pytest.xfail(f'The current device {idx} network is abnormal, please check it and run it again!')
                    if agent:
                        proxy(devices=idx, status=True)

                if init:
                    if str(uri).startswith('iOS'):
                        os.system('utx uninstall {}'.format(app_name))
                        os.system('utx install {}'.format(app_pkg))
                        UT.iOS = True
                    else:
                        try:
                            uninstall(app_name)
                        except AdbError:
                            pass
                        install(app_pkg, install_options=["-g"])
                    stop_app(app_name)
                    sleep(2)
                    start_app(app_name)
                    sleep(2)
                    UT.LOOP = 2
                    UT.SYS = True
                    popup(devices=uri)
                else:
                    stop_app(app_name)
                    sleep(2)
                    start_app(app_name)
                    sleep(2)
        except Exception as e:

            if device_platform().lower() in 'ios':
                cleanup()

            if device_platform().lower() in 'android' and agent:
                for _ in device_idx:
                    proxy(devices=_, status=False)

            logger.error("Create app fail: {}".format(e))
            allure.attach(body='',
                          name="Create app fail: {}".format(e),
                          attachment_type=allure.attachment_type.TEXT)
            pytest.exit("Create app fail: {}".format(e))

        assert (app is not None)

        ensure_current_device(device_idx[0])

        logger.info("Current test platform: {}".format(device_platform()))
        logger.info("Start app {0} in {1}:{2}".format(app_name, device_platform(), G.DEVICE.uuid))

    def teardown_test():
        with allure.step("Teardown session"):
            stop_app(app_name)
            # todo: Do not uninstall the installation package at the end of the execution
            #  which is convenient for troubleshooting.

            if device_platform().lower() in 'ios':
                check_port(port=str(device_idx[0]).split(':')[1])
                cleanup()
                logger.info("Cleanup device wda process.")
            if device_platform().lower() in 'android' and agent:
                for _ in device_idx:
                    proxy(devices=_, status=False)
            keyevent('26')

        logger.info("Session stop test.")

    request.addfinalizer(teardown_test)

    return app


def web_info(cli_headless, ini_headless):
    """

    :param cli_headless:
    :param ini_headless:
    :return:
    """

    if cli_headless is None:
        is_headless = str_to_bool(ini_headless)
    else:
        is_headless = str_to_bool(cli_headless)

    return is_headless


driver = None


def web_fixture(request, headless):
    """

    :param request:
    :param headless:
    :return:
    """
    with allure.step("Initialize and generate Web object！"):
        logger.info("Session start test.")

    global driver
    if driver is None:
        chrome_options = Options()
        if plat() == 'Windows':
            chrome_options.add_argument("--remote-debugging-port=9222")
            if headless:
                chrome_options.add_argument('--headless')
        elif plat() == 'Linux':
            chrome_options.add_argument("--remote-debugging-port=9222")
            chrome_options.add_argument('--headless')
            chrome_options.add_argument('--no-sandbox')
            chrome_options.add_argument('--disable-dev-shm-usage')
        else:
            chrome_options.add_argument("--remote-debugging-port=9222")
            if headless:
                chrome_options.add_argument('--headless')

        driver = webdriver.Chrome(options=chrome_options,
                                  executable_path=ChromeDriverManager().install())

        driver.maximize_window()
        # driver.set_window_size(1920, 1080)

    driver.implicitly_wait(30)

    def fn():
        with allure.step("Teardown session"):
            driver.quit()
        logger.info("Session stop test.")

    request.addfinalizer(fn)
    return driver



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  2021/12/11 11:14 上午 
@Desc    :  pg_1 line.
"""
from utx.mod.platform import PluginsProcessor


@PluginsProcessor.plugin_register('pg1')
class CleanMarkdownBolds(object):
    @staticmethod
    def process(text):
        return text.replace('**', '')



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  8/31/2021 6:42 PM
@Desc    :  Runner line.
"""
import time
import unittest
import sys
import six
import re
import shutil
import traceback
import warnings
from io import open

from popups.dismiss import popup, UT
from airtest.core.api import G, auto_setup, log, stop_app, shell, connect_device
from airtest.core.helper import device_platform
from airtest.core.settings import Settings as ST
from airtest.utils.compat import script_dir_name, script_log_dir
from copy import copy

from argparse import *

from airtest.core.android.adb import ADB
from airtest.core.api import device, start_app
from airtest.core.error import AirtestError

from utx.core.utils.tools import ios_device_info, plat, decryption
from poco.exceptions import PocoException

from utx.core.css import report
import os


class AirtestCase(unittest.TestCase):
    PROJECT_ROOT = "."
    SCRIPTEXT = ".air"
    TPLEXT = ".png"

    @classmethod
    def setUpClass(cls):
        cls.args = args

        setup_by_args(args)

        # setup script exec scope
        cls.scope = copy(globals())
        cls.scope["exec_script"] = cls.exec_other_script

    def setUp(self):
        if self.args.log and self.args.recording:
            try:
                device().start_recording()
            except AirtestError:
                traceback.print_exc()

    def tearDown(self):
        if self.args.log and self.args.recording:
            try:
                if isinstance(self.args.recording, six.string_types) and self.args.recording.endswith(".mp4"):
                    output_name = os.path.basename(self.args.recording)
                    # output_name = dev.serialno + "_" + basename if len(G.DEVICE_LIST) > 1 else basename
                else:
                    output_name = "recording_{serialno}.mp4".format(serialno=device().serialno)
                output = os.path.join(self.args.log, output_name)
                device().stop_recording(output)
            except AirtestError:
                traceback.print_exc()

    def runTest(self):
        scriptpath, pyfilename = script_dir_name(self.args.script)
        pyfilepath = os.path.join(scriptpath, pyfilename)
        pyfilepath = os.path.abspath(pyfilepath)
        self.scope["__file__"] = pyfilepath
        with open(pyfilepath, 'r', encoding="utf8") as f:
            code = f.read()
        pyfilepath = pyfilepath.encode(sys.getfilesystemencoding())

        try:
            exec(compile(code.encode("utf-8"), pyfilepath, 'exec'), self.scope)
        except Exception as err:
            log(err, desc="Final Error", snapshot=True)
            six.reraise(*sys.exc_info())

    @classmethod
    def exec_other_script(cls, scriptpath):
        """run other script in test script"""

        warnings.simplefilter("always")
        warnings.warn("please use using() api instead.", PendingDeprecationWarning)

        def _sub_dir_name(scriptname):
            dirname = os.path.splitdrive(os.path.normpath(scriptname))[-1]
            dirname = dirname.strip(os.path.sep).replace(os.path.sep, "_").replace(cls.SCRIPTEXT, "_sub")
            return dirname

        def _copy_script(src, dst):
            if os.path.isdir(dst):
                shutil.rmtree(dst, ignore_errors=True)
            os.mkdir(dst)
            for f in os.listdir(src):
                srcfile = os.path.join(src, f)
                if not (os.path.isfile(srcfile) and f.endswith(cls.TPLEXT)):
                    continue
                dstfile = os.path.join(dst, f)
                shutil.copy(srcfile, dstfile)

        # find script in PROJECT_ROOT
        scriptpath = os.path.join(ST.PROJECT_ROOT, scriptpath)
        # copy submodule's images into sub_dir
        sub_dir = _sub_dir_name(scriptpath)
        sub_dirpath = os.path.join(cls.args.script, sub_dir)
        _copy_script(scriptpath, sub_dirpath)
        # read code
        pyfilename = os.path.basename(scriptpath).replace(cls.SCRIPTEXT, ".py")
        pyfilepath = os.path.join(scriptpath, pyfilename)
        pyfilepath = os.path.abspath(pyfilepath)
        with open(pyfilepath, 'r', encoding='utf8') as f:
            code = f.read()
        # replace tpl filepath with filepath in sub_dir
        code = re.sub(r"[\'\"](\w+.png)[\'\"]", r"\"%s/\g<1>\"" % sub_dir, code)
        exec(compile(code.encode("utf8"), pyfilepath, 'exec'), cls.scope)


def setup_by_args(args):
    # init devices
    if isinstance(args.device, list):
        devices = args.device
    elif args.device:
        devices = [args.device]
    else:
        devices = []
        print("do not connect device")

    # set base dir to find tpl
    dirpath, _ = script_dir_name(args.script)

    # set log dir
    if args.log:
        args.log = script_log_dir(dirpath, args.log)
        print("save log in '%s'" % args.log)
    else:
        print("do not save log")

    # set snapshot quality
    if args.compress:
        compress = args.compress
    else:
        compress = ST.SNAPSHOT_QUALITY

    if args.no_image:
        ST.SAVE_IMAGE = False

    # guess project_root to be basedir of current .air path
    project_root = os.path.dirname(args.script) if not ST.PROJECT_ROOT else None

    auto_setup(dirpath, devices, args.log, project_root, compress)


def run_script(parsed_args, testcase_cls=AirtestCase):
    global args  # make it global deliberately to be used in AirtestCase & test scripts
    args = parsed_args
    suite = unittest.TestSuite()
    suite.addTest(testcase_cls())
    result = unittest.TextTestRunner(verbosity=0).run(suite)
    # if not result.wasSuccessful():
    #     sys.exit(-1)


def custom_resize_method(w, h, sch_resolution, src_resolution):
    """
    Resolution Adaptation Rules
    :param w:
    :param h:
    :param sch_resolution:
    :param src_resolution:
    :return: device resolution
    """
    return int(w), int(h)


def run_air(devices, case, app_name, log_path, case_path, base_dir,
            static_dir=decryption('aHR0cHM6Ly9mYXN0bHkuanNkZWxpdnIubmV0L2doL29wZW51dHgvc3RhdGljLw=='), record=None):
    """
    //See also: https://www.theiphonewiki.com/wiki/Models

    :param devices: read device id
    :param case: case list
    :param record: whether to enable recording
    :param log_path: log_path
    :param case_path: case_path
    :param base_dir: base_dir
    :param app_name: app_name
    :param static_dir: use local style file when empty

    :return:
    """

    connect_device(devices)

    if device_platform().lower() in 'ios':
        _device = ios_device_info()
        model = _device['MarketName']
        build_version = _device['ProductVersion']
        dev = model + ' ios {}'.format(build_version).replace('\n', '').replace('\r', '')

    elif device_platform().lower() in 'android':
        serialno = device().serialno
        model = ADB(serialno=serialno).get_model()
        build_version = shell('getprop ro.build.version.release')
        dev = model + ' Android {}'.format(build_version).replace('\n', '').replace('\r', '')
    else:
        print('{} is unsupported platform on utx!'.format(device_platform()))
        raise AirtestError('utx unsupported this platform!')

    UT.SYS = False
    UT.LOOP = 1
    popup(devices=devices)

    if os.path.isdir(log_path):
        pass
    else:
        os.makedirs(log_path)
        print(str(log_path) + 'is created')

    if str(case).endswith(".air"):
        if plat() == 'Windows':
            air_name = str(case).split('\\')[-1]
        else:
            air_name = str(case).split('/')[-1]

        if record:
            record = air_name.replace('.air', '.mp4')

        print(case)
        log = os.path.join(log_path + '/' + dev + '/' + air_name.replace('.air', ''))

        run_log = case + '/' + 'log' + '/' + dev
        if not os.path.exists(run_log):
            os.makedirs(run_log)
        print(log)
        if os.path.isdir(log):
            pass
        else:
            os.makedirs(log)
            print(str(log) + 'is created')
        output_file = log + '/' + 'log.html'
        args = Namespace(device=devices, log=run_log, recording=record, script=case, compress=20, no_image=False)
        try:
            run_script(args, AirtestCase)
        except (AirtestError, PocoException):
            pass
        finally:
            rpt = report.LogToHtml(case, run_log, script_name=air_name.replace(".air", ".py"), static_root=static_dir,
                                   plugins=['utx.mod.report.seleniumui', 'utx.mod.report.pocoui'])
            rpt.report("log_template.html", output_file=output_file, base_dir=base_dir)
            result = {"name": air_name.replace('.air', ''), "result": rpt.test_result}

        stop_app(app_name)
        time.sleep(2)
        start_app(app_name)
        time.sleep(2)

        return result, dev


def run_web_air(case, log_path, case_path, base_dir,
                static_dir=decryption('aHR0cHM6Ly9mYXN0bHkuanNkZWxpdnIubmV0L2doL29wZW51dHgvc3RhdGljLw==')):
    """

    :param case:
    :param log_path:
    :param case_path:
    :param base_dir:
    :param static_dir:
    :return:
    """
    dev = 'chrome'
    time.sleep(2)

    if os.path.isdir(log_path):
        pass
    else:
        os.makedirs(log_path)
        print(str(log_path) + 'is created')

    if str(case).endswith(".air"):
        if plat() == 'Windows':
            air_name = str(case).split('\\')[-1]
        else:
            air_name = str(case).split('/')[-1]
        print(case)
        log = os.path.join(log_path + '/' + dev + '/' + air_name.replace('.air', ''))

        run_log = case + '/' + 'log' + '/' + dev
        if not os.path.exists(run_log):
            os.makedirs(run_log)
        print(log)
        if os.path.isdir(log):
            pass
        else:
            os.makedirs(log)
            print(str(log) + 'is created')
        output_file = log + '/' + 'log.html'
        args = Namespace(device=None, log=run_log, recording=None, script=case, compress=20, no_image=False)
        try:
            run_script(args, AirtestCase)
        except Exception:
            pass
        finally:
            rpt = report.LogToHtml(case, run_log, script_name=air_name.replace(".air", ".py"), static_root=static_dir,
                                   plugins=['utx.mod.report.seleniumui', 'utx.mod.report.pocoui'])
            rpt.report("log_template.html", output_file=output_file, base_dir=base_dir)
            result = {"name": air_name.replace('.air', ''), "result": rpt.test_result}

        return result, dev



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  2022/2/8 10:58 上午
@Desc    :  ipa_pkg line.
"""
import os
import shutil

from utx.core.utils.tools import decryption


def pkg(wda_home, ipa_home):
    """
    mac 打包utx.ipa包
    :param wda_home: 克隆到本地 https://github.com/openutx/utx-wda.git
    :param ipa_home: ipa目录
    :return:
    """
    if os.path.exists(ipa_home):
        shutil.rmtree(ipa_home)
    os.makedirs(ipa_home)
    cmd = f"""
    cd {wda_home} &&
    xcodebuild build-for-testing -scheme WebDriverAgentRunner -sdk iphoneos -configuration Release -derivedDataPath {ipa_home}
    cd {ipa_home}/Build/Products/Release-iphoneos
    mkdir Payload && mv *.app Payload
    zip -r UTX-Runner.ipa Payload
    mv *.ipa {os.getcwd()}
    """
    os.system(cmd)
    os.system('{} parse UTX-Runner.ipa'.format(decryption(b'dGlkZXZpY2U=')))



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  11/02/2021 3:36 PM
@Desc    :  proxy line
"""

from selenium.webdriver import Chrome, ActionChains, Firefox, Remote
from selenium.webdriver.remote.webelement import WebElement
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.chrome.options import Options
from webdriver_manager.chrome import ChromeDriverManager
from airtest.core.settings import Settings as ST
from airtest.core.helper import logwrap
from airtest import aircv
from airtest.core.cv import Template
from utx.selenium.utils.airtest_api import loop_find
from utx.selenium.exceptions import IsNotTemplateError
from airtest.aircv import get_resolution
from pynput.mouse import Controller, Button
from airtest.core.error import TargetNotFoundError

import os
import time
import sys


class WebChrome(Chrome):
    os.environ['WDM_SSL_VERIFY'] = '0'
    chrome_options = Options()
    chrome_options.add_experimental_option("debuggerAddress", "127.0.0.1:9222")

    def __init__(self, executable_path=ChromeDriverManager().install(), port=0,
                 options=chrome_options, service_args=None,
                 desired_capabilities=None, service_log_path=None,
                 chrome_options=None):

        # todo 适配selenium 4.0.0
        # def __init__(self, executable_path="chromedriver", port=0,
        #              options: Options = chrome_options, service_args=None,
        #              desired_capabilities=None, service_log_path=None,
        #              chrome_options=None, service=Service(executable_path=driver_path), keep_alive=None):

        if "darwin" in sys.platform:
            os.environ['PATH'] += ":/Applications/AirtestIDE.app/Contents/Resources/selenium_plugin"
        super(WebChrome, self).__init__(chrome_options=chrome_options, executable_path=executable_path,
                                        port=port, options=options, service_args=service_args,
                                        service_log_path=service_log_path, desired_capabilities=desired_capabilities)
        self.father_number = {0: 0}
        self.action_chains = ActionChains(self)
        self.number = 0
        self.mouse = Controller()
        self.operation_to_func = {"elementsD": self.find_any_element, "xpath": self.find_element_by_xpath,
                                  "id": self.find_element_by_id,
                                  "name": self.find_element_by_name, "css": self.find_element_by_css_selector}

    def loop_find_element(self, func, text, timeout=10, interval=0.5):
        """
        Loop to find the target web element by func.

        Args:
            func: function to find element
            text: param of function
            timeout: time to find the element
            interval: interval between operation
        Returns:
            element that been found
        """
        start_time = time.time()
        while True:
            try:
                element = func(text)
            except NoSuchElementException:
                print("Element not found!")
                # 超时则raise，未超时则进行下次循环:
                if (time.time() - start_time) > timeout:
                    # try_log_screen(screen)
                    raise NoSuchElementException(
                        'Element %s not found in screen' % text)
                else:
                    time.sleep(interval)
            else:
                return element

    def loop_find_element_noExc(self, func, text, timeout=3, interval=0.5):
        """
        Loop to find the target web element by func.

        Args:
            func: function to find element
            text: param of function
            timeout: time to find the element
            interval: interval between operation
        Returns:
            element that been found
        """
        start_time = time.time()
        while True:
            try:
                element = func(text)
            except NoSuchElementException:
                print("Element not found!")
                # 超时则raise，未超时则进行下次循环:
                if (time.time() - start_time) > timeout:
                    # try_log_screen(screen)
                    return None
                else:
                    time.sleep(interval)
            else:
                print('element found')
                return element

    @logwrap
    def find_any_element(self, elementsD):
        """
        Find the web element by the indicated parameters.

        Args:
            elementsD: a dictionary with keys = by's, values=value
        Returns:
            Web element of current page.
        """
        web_element = None
        for key in elementsD:
            value = elementsD[key]
            print(value)
            if key.upper() == 'ID':
                web_element = self.loop_find_element_noExc(super().find_element_by_id, value)
            elif key.upper() == 'XPATH':
                web_element = self.loop_find_element_noExc(super().find_element_by_xpath, value)
            elif key.upper() == 'CSS':
                web_element = self.loop_find_element_noExc(super().find_element_by_css_selector, value)
            elif key.upper() == 'NAME':
                web_element = self.loop_find_element_noExc(super().find_element_by_name, value)
            elif key.upper() == 'LINKTEXT':
                web_element = self.loop_find_element_noExc(super().find_element_by_link_text, value)
            elif key.upper() == 'CLASSNAME':
                web_element = self.loop_find_element_noExc(super().find_element_by_class_name, value)
            elif key.upper() == 'PARTIALLINKTEXT':
                web_element = self.loop_find_element_noExc(super().find_element_by_partial_link_text, value)
            elif key.upper() == 'TAGNAME':
                web_element = self.loop_find_element_noExc(super().find_element_by_tag_name, value)
                # check by position/ picture / visual testing
            if web_element is not None:
                break
        if web_element is not None:
            log_res = self._gen_screen_log(web_element)
            return Element(web_element, log_res)
        raise NoSuchElementException('Element not found in screen')

    @logwrap
    def find_element_by_xpath(self, xpath):
        """
        Find the web element by xpath.

        Args:
            xpath: find the element by xpath.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebChrome, self).find_element_by_xpath, xpath)
        # web_element = super(WebChrome, self).find_element_by_xpath(xpath)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_id(self, id):
        """
        Find the web element by id.

        Args:
            id: find the element by attribute id.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebChrome, self).find_element_by_id, id)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_css_selector(self, css_selector):
        """
        Find the web element by css_selector.

        Args:
            css_selector: find the element by attribute css_selector.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebChrome, self).find_element_by_css_selector, css_selector)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_name(self, name):
        """
        Find the web element by name.

        Args:
            name: find the element by attribute name.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebChrome, self).find_element_by_name, name)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def switch_to_new_tab(self):
        """
        Switch to the new tab.
        """
        _father = self.number
        self.number = len(self.window_handles) - 1
        self.father_number[self.number] = _father
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def switch_to_previous_tab(self):
        """
        Switch to the previous tab(which to open current tab).
        """
        self.number = self.father_number[self.number]
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def airtest_touch(self, v):
        """
        Perform the touch action on the current page by image identification.

        Args:
            v: target to touch, either a Template instance or absolute coordinates (x, y)
        Returns:
            Finial position to be clicked.
        """
        # 调用airtest opencv 图像识别能力
        if isinstance(v, Template):
            _pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
        else:
            _pos = v
        x, y = _pos
        # self.action_chains.move_to_element_with_offset(root_element, x, y)
        # self.action_chains.click()
        pos = self._get_left_up_offset()
        pos = (pos[0] + x, pos[1] + y)
        self._move_to_pos(pos)
        self._click_current_pos()
        time.sleep(1)
        return _pos

    @logwrap
    def assert_template(self, v, msg=""):
        """
        Assert target exists on the current page.

        Args:
            v: target to touch, either a Template instance
            msg: assert messages
        Raise:
            AssertionError - if target not found.
        Returns:
            Position of the template.
        """
        if isinstance(v, Template):
            try:
                pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
            except TargetNotFoundError:
                raise AssertionError("Target template not found on screen.")
            else:
                return pos
        else:
            raise IsNotTemplateError("args is not a template")

    @logwrap
    def assert_exist(self, param, operation, msg=""):
        """
        Assert element exist.

        Args:
            operation: the method that to find the element.
            param: the param of method.
            msg: assert messages
        Raise:
            AssertionError - if assertion failed.
        """
        try:
            func = self.operation_to_func[operation]
        except Exception:
            raise AssertionError("There was no operation: %s" % operation)
        try:
            func(param)
        except Exception as e:
            raise AssertionError("Target element not find.")

    @logwrap
    def get(self, address):
        """
        Access the web address.

        Args:
            address: the address that to accesss
        """
        super(WebChrome, self).get(address)
        time.sleep(2)

    @logwrap
    def back(self):
        """
        Back to last page.
        """
        super(WebChrome, self).back()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def forward(self):
        """
        Forward to next page.
        """
        super(WebChrome, self).forward()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def snapshot(self, filename=None):
        self._gen_screen_log(filename=filename)

    @logwrap
    def _gen_screen_log(self, element=None, filename=None, ):
        if ST.LOG_DIR is None:
            return None
        if filename:
            self.screenshot(filename)

        # 忽略警告 .jpg
        jpg_file_name = str(int(time.time())) + '.png'
        # 适配utx报告路径
        jpg_path = os.path.join(ST.LOG_DIR, jpg_file_name)
        print("this is jpg path:", jpg_path)
        self.screenshot(jpg_path)
        saved = {"screen": jpg_path}
        if element:
            size = element.size
            location = element.location
            x = size['width'] / 2 + location['x']
            y = size['height'] / 2 + location['y']
            if "darwin" in sys.platform:
                x, y = x * 2, y * 2
            saved.update({"pos": [[x, y]]})
        return saved

    def screenshot(self, file_path=None):
        if file_path:
            self.save_screenshot(file_path)
        else:
            if not ST.LOG_DIR:
                file_path = "temp.png"
            else:
                file_path = os.path.join(ST.LOG_DIR, "temp.png")
            self.save_screenshot(file_path)
            screen = aircv.imread(file_path)
            return screen

    def _get_left_up_offset(self):
        window_pos = self.get_window_position()
        window_size = self.get_window_size()
        mouse = Controller()
        screen = self.screenshot()
        screen_size = get_resolution(screen)
        offset = window_size["width"] - \
                 screen_size[0], window_size["height"] - screen_size[1]
        pos = (int(offset[0] / 2 + window_pos['x']),
               int(offset[1] + window_pos['y'] - offset[0] / 2))
        return pos

    def _move_to_pos(self, pos):
        self.mouse.position = pos

    def _click_current_pos(self):
        self.mouse.click(Button.left, 1)

    def to_json(self):
        # add this method for json encoder in logwrap
        return repr(self)


class WebRemote(Remote):

    def __init__(self, command_executor='http://127.0.0.1:4444/wd/hub',
                 desired_capabilities=None, browser_profile=None, proxy=None,
                 keep_alive=False, file_detector=None, options=None):
        super(WebRemote, self).__init__(command_executor=command_executor,
                                        desired_capabilities=desired_capabilities, browser_profile=browser_profile,
                                        proxy=proxy,
                                        keep_alive=keep_alive, file_detector=file_detector, options=options)

        self.father_number = {0: 0}
        self.action_chains = ActionChains(self)
        self.number = 0
        self.mouse = Controller()
        self.operation_to_func = {"xpath": self.find_element_by_xpath, "id": self.find_element_by_id,
                                  "name": self.find_element_by_name, "css": self.find_element_by_css_selector}

    def loop_find_element(self, func, text, timeout=10, interval=0.5):
        """
        Loop to find the target web element by func.

        Args:
            func: function to find element
            text: param of function
            timeout: time to find the element
            interval: interval between operation
        Returns:
            element that been found
        """
        start_time = time.time()
        while True:
            try:
                element = func(text)
            except NoSuchElementException:
                print("Element not found!")
                # 超时则raise，未超时则进行下次循环:
                if (time.time() - start_time) > timeout:
                    # try_log_screen(screen)
                    raise NoSuchElementException('Element %s not found in screen' % text)
                else:
                    time.sleep(interval)
            else:
                return element

    @logwrap
    def find_element_by_xpath(self, xpath):
        """
        Find the web element by xpath.

        Args:
            xpath: find the element by xpath.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebRemote, self).find_element_by_xpath, xpath)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_id(self, id):
        """
        Find the web element by id.

        Args:
            id: find the element by attribute id.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebRemote, self).find_element_by_id, id)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_css_selector(self, css_selector):
        """
        Find the web element by css_selector.

        Args:
            css_selector: find the element by attribute css_selector.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebRemote, self).find_element_by_css_selector, css_selector)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_name(self, name):
        """
        Find the web element by name.

        Args:
            name: find the element by attribute name.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebRemote, self).find_element_by_name, name)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def switch_to_new_tab(self):
        """
        Switch to the new tab.
        """
        _father = self.number
        self.number = len(self.window_handles) - 1
        self.father_number[self.number] = _father
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def switch_to_previous_tab(self):
        """
        Switch to the previous tab(which to open current tab).
        """
        self.number = self.father_number[self.number]
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def airtest_touch(self, v):
        """
        Perform the touch action on the current page by image identification.

        Args:
            v: target to touch, either a Template instance or absolute coordinates (x, y)
        Returns:
            Finial position to be clicked.
        """
        if isinstance(v, Template):
            _pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
        else:
            _pos = v
        x, y = _pos
        # self.action_chains.move_to_element_with_offset(root_element, x, y)
        # self.action_chains.click()
        pos = self._get_left_up_offset()
        pos = (pos[0] + x, pos[1] + y)
        self._move_to_pos(pos)
        self._click_current_pos()
        time.sleep(1)
        return _pos

    @logwrap
    def assert_template(self, v, msg=""):
        """
        Assert target exists on the current page.

        Args:
            v: target to touch, either a Template instance
        Raise:
            AssertionError - if target not found.
        Returns:
            Position of the template.
        """
        if isinstance(v, Template):
            try:
                pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
            except TargetNotFoundError:
                raise AssertionError("Target template not found on screen.")
            else:
                return pos
        else:
            raise IsNotTemplateError("args is not a template")

    @logwrap
    def assert_exist(self, param, operation, msg=""):
        """
        Assert element exist.

        Args:
            operation: the method that to find the element.
            param: the param of method.
        Raise:
            AssertionError - if assertion failed.
        """
        try:
            func = self.operation_to_func[operation]
        except Exception:
            raise AssertionError("There was no operation: %s" % operation)
        try:
            func(param)
        except Exception as e:
            raise AssertionError("Target element not find.")

    @logwrap
    def get(self, address):
        """
        Access the web address.

        Args:
            address: the address that to accesss
        """
        super(WebRemote, self).get(address)
        time.sleep(2)

    @logwrap
    def back(self):
        """
        Back to last page.
        """
        super(WebRemote, self).back()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def forward(self):
        """
        Forward to next page.
        """
        super(WebRemote, self).forward()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def snapshot(self, filename=None):
        self._gen_screen_log(filename=filename)

    @logwrap
    def _gen_screen_log(self, element=None, filename=None, ):
        if ST.LOG_DIR is None:
            return None
        if filename:
            self.screenshot(filename)
        jpg_file_name = str(int(time.time())) + '.png'
        jpg_path = os.path.join(ST.LOG_DIR, jpg_file_name)
        print("this is jpg path:", jpg_path)
        self.screenshot(jpg_path)
        saved = {"screen": jpg_path}
        if element:
            size = element.size
            location = element.location
            x = size['width'] / 2 + location['x']
            y = size['height'] / 2 + location['y']
            if "darwin" in sys.platform:
                x, y = x * 2, y * 2
            saved.update({"pos": [[x, y]]})
        return saved

    def screenshot(self, file_path=None):
        if file_path:
            self.save_screenshot(file_path)
        else:
            if not ST.LOG_DIR:
                file_path = "temp.png"
            else:
                file_path = os.path.join(ST.LOG_DIR, "temp.png")
            self.save_screenshot(file_path)
            screen = aircv.imread(file_path)
            return screen

    def _get_left_up_offset(self):
        window_pos = self.get_window_position()
        window_size = self.get_window_size()
        mouse = Controller()
        screen = self.screenshot()
        screen_size = get_resolution(screen)
        offset = window_size["width"] - \
                 screen_size[0], window_size["height"] - screen_size[1]
        pos = (int(offset[0] / 2 + window_pos['x']),
               int(offset[1] + window_pos['y'] - offset[0] / 2))
        return pos

    def _move_to_pos(self, pos):
        self.mouse.position = pos

    def _click_current_pos(self):
        self.mouse.click(Button.left, 1)

    def to_json(self):
        # add this method for json encoder in logwrap
        return repr(self)


class WebFirefox(Firefox):

    def __init__(self, firefox_profile=None, firefox_binary=None,
                 timeout=30, capabilities=None, proxy=None,
                 executable_path="geckodriver", options=None, firefox_options=None,
                 service_args=None, desired_capabilities=None, log_path=None):
        print("Please make sure your geckodriver is in your path before proceeding using this driver")
        super(WebFirefox, self).__init__(firefox_profile=firefox_profile, firefox_binary=firefox_binary,
                                         timeout=timeout, capabilities=capabilities, proxy=proxy,
                                         executable_path=executable_path, options=options,
                                         firefox_options=firefox_options,
                                         service_args=service_args, desired_capabilities=desired_capabilities,
                                         log_path=log_path)
        self.father_number = {0: 0}
        self.action_chains = ActionChains(self)
        self.number = 0
        self.mouse = Controller()
        self.operation_to_func = {"xpath": self.find_element_by_xpath, "id": self.find_element_by_id,
                                  "name": self.find_element_by_name, "css": self.find_element_by_css_selector}

    def loop_find_element(self, func, text, timeout=10, interval=0.5):
        """
        Loop to find the target web element by func.

        Args:
            func: function to find element
            text: param of function
            timeout: time to find the element
            interval: interval between operation
        Returns:
            element that been found
        """
        start_time = time.time()
        while True:
            try:
                element = func(text)
            except NoSuchElementException:
                print("Element not found!")
                # 超时则raise，未超时则进行下次循环:
                if (time.time() - start_time) > timeout:
                    # try_log_screen(screen)
                    raise NoSuchElementException('Element %s not found in screen' % text)
                else:
                    time.sleep(interval)
            else:
                return element

    @logwrap
    def find_element_by_xpath(self, xpath):
        """
        Find the web element by xpath.

        Args:
            xpath: find the element by xpath.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(Firefox, self).find_element_by_xpath, xpath)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_id(self, id):
        """
        Find the web element by id.

        Args:
            id: find the element by attribute id.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(Firefox, self).find_element_by_id, id)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_css_selector(self, css_selector):
        """
        Find the web element by css_selector.

        Args:
            css_selector: find the element by attribute css_selector.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(Firefox, self).find_element_by_css_selector, css_selector)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_name(self, name):
        """
        Find the web element by name.

        Args:
            name: find the element by attribute name.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(Firefox, self).find_element_by_name, name)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def switch_to_new_tab(self):
        """
        Switch to the new tab.
        """
        _father = self.number
        self.number = len(self.window_handles) - 1
        self.father_number[self.number] = _father
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def switch_to_previous_tab(self):
        """
        Switch to the previous tab(which to open current tab).
        """
        self.number = self.father_number[self.number]
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def airtest_touch(self, v):
        """
        Perform the touch action on the current page by image identification.

        Args:
            v: target to touch, either a Template instance or absolute coordinates (x, y)
        Returns:
            Finial position to be clicked.
        """
        if isinstance(v, Template):
            _pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
        else:
            _pos = v
        x, y = _pos
        # self.action_chains.move_to_element_with_offset(root_element, x, y)
        # self.action_chains.click()
        pos = self._get_left_up_offset()
        pos = (pos[0] + x, pos[1] + y)
        self._move_to_pos(pos)
        self._click_current_pos()
        time.sleep(1)
        return _pos

    @logwrap
    def assert_template(self, v, msg=""):
        """
        Assert target exists on the current page.

        Args:
            v: target to touch, either a Template instance
        Raise:
            AssertionError - if target not found.
        Returns:
            Position of the template.
        """
        if isinstance(v, Template):
            try:
                pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
            except TargetNotFoundError:
                raise AssertionError("Target template not found on screen.")
            else:
                return pos
        else:
            raise IsNotTemplateError("args is not a template")

    @logwrap
    def assert_exist(self, param, operation, msg=""):
        """
        Assert element exist.

        Args:
            operation: the method that to find the element.
            param: the param of method.
        Raise:
            AssertionError - if assertion failed.
        """
        try:
            func = self.operation_to_func[operation]
        except Exception:
            raise AssertionError("There was no operation: %s" % operation)
        try:
            func(param)
        except Exception as e:
            raise AssertionError("Target element not find.")

    @logwrap
    def get(self, address):
        """
        Access the web address.

        Args:
            address: the address that to accesss
        """
        super(WebFirefox, self).get(address)
        time.sleep(2)

    @logwrap
    def back(self):
        """
        Back to last page.
        """
        super(WebFirefox, self).back()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def forward(self):
        """
        Forward to next page.
        """
        super(WebFirefox, self).forward()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def snapshot(self, filename=None):
        self._gen_screen_log(filename=filename)

    @logwrap
    def _gen_screen_log(self, element=None, filename=None, ):
        if ST.LOG_DIR is None:
            return None
        if filename:
            self.screenshot(filename)
        jpg_file_name = str(int(time.time())) + '.png'
        jpg_path = os.path.join(ST.LOG_DIR, jpg_file_name)
        print("this is jpg path:", jpg_path)
        self.screenshot(jpg_path)
        saved = {"screen": jpg_path}
        if element:
            size = element.size
            location = element.location
            x = size['width'] / 2 + location['x']
            y = size['height'] / 2 + location['y']
            if "darwin" in sys.platform:
                x, y = x * 2, y * 2
            saved.update({"pos": [[x, y]]})
        return saved

    def screenshot(self, file_path=None):
        if file_path:
            self.save_screenshot(file_path)
        else:
            if not ST.LOG_DIR:
                file_path = "temp.png"
            else:
                file_path = os.path.join(ST.LOG_DIR, "temp.png")
            self.save_screenshot(file_path)
            screen = aircv.imread(file_path)
            return screen

    def _get_left_up_offset(self):
        window_pos = self.get_window_position()
        window_size = self.get_window_size()
        mouse = Controller()
        screen = self.screenshot()
        screen_size = get_resolution(screen)
        offset = window_size["width"] - \
                 screen_size[0], window_size["height"] - screen_size[1]
        pos = (int(offset[0] / 2 + window_pos['x']),
               int(offset[1] + window_pos['y'] - offset[0] / 2))
        return pos

    def _move_to_pos(self, pos):
        self.mouse.position = pos

    def _click_current_pos(self):
        self.mouse.click(Button.left, 1)

    def to_json(self):
        # add this method for json encoder in logwrap
        return repr(self)


class Element(WebElement):

    def __init__(self, _obj, log):
        super(Element, self).__init__(
            parent=_obj._parent, id_=_obj._id, w3c=_obj._w3c)
        self.res_log = log

    def click(self):
        super(Element, self).click()
        time.sleep(0.5)
        return self.res_log

    def send_keys(self, text, keyborad=None):
        if keyborad:
            super(Element, self).send_keys(text, keyborad)
        else:
            super(Element, self).send_keys(text)
        time.sleep(0.5)
        return self.res_log



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  9/14/2021 4:36 PM
@Desc    :  Custom Report line.
"""
import io
import os
import jinja2

CUSTOM_HTML_TPL = "summary_template_v2.html"
CUSTOM_STATIC_DIR = os.path.dirname(__file__)


def gen_report(results=None, report_path=None, report_name='utx_summary.html'):
    """
    gen_report
    :param report_name: custom name
    :param results: results list
    :param report_path: report_path
    :return: custom report html
    """
    format_list = []
    for i in results:
        if i not in format_list:
            format_list.append(i)

    for f in format_list:
        result = []
        for res in f['result']:
            if res not in result:
                result.append(res)
        f['result'] = result

    env = jinja2.Environment(
        loader=jinja2.FileSystemLoader(CUSTOM_STATIC_DIR),
        extensions=(),
        autoescape=True
    )
    template = env.get_template(CUSTOM_HTML_TPL, CUSTOM_STATIC_DIR)
    html = template.render({"results": format_list})
    output_file = os.path.join(report_path, report_name)
    with io.open(output_file, 'w', encoding="utf-8") as f:
        f.write(html)
    print(output_file)



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  8/31/2021 9:42 PM
@Desc    :  Tools line.
"""
import base64
import platform
import socket
import subprocess
import re
import configparser
import requests
import urllib3
from multiprocessing import Process
from airtest.core.android.recorder import Recorder
from airtest.core.error import AdbShellError
from loguru import logger
from tqdm import tqdm
from airtest.utils.logger import get_logger
from airtest.core.api import *
from airtest.core.android.adb import ADB

LOGGING = get_logger(__name__)


def allure_report(report_path, report_html):
    """
    Generate allure Report
    :param report_path:
    :param report_html:
    :return:
    """
    # execution allure generate
    allure_cmd = "allure generate %s -o %s --clean" % (report_path, report_html)
    try:
        subprocess.call(allure_cmd, shell=True)
    except Exception:
        LOGGING.error("The generation of allure report failed. Please check the relevant configuration of the test "
                      "environment")
        raise


def plat():
    """
    Check the current script running platform
    :return:'Linux', 'Windows' or 'Darwin'.
    """
    return platform.system()


def check_port(port):
    """
    Detect whether the port is occupied and clean up
    :param port:System port
    :return:None
    """
    if plat() != 'Windows':
        os.system("lsof -i:%s| grep LISTEN| awk '{print $2}'|xargs kill -9" % port)
    else:
        port_cmd = 'netstat -ano | findstr {}'.format(port)
        r = os.popen(port_cmd)
        if len(r.readlines()) == 0:
            return
        else:
            pid_list = []
            for line in r.readlines():
                line = line.strip()
                pid = re.findall(r'[1-9]\d*', line)
                pid_list.append(pid[-1])
            pid_set = list(set(pid_list))[0]
            pid_cmd = 'taskkill -PID {} -F'.format(pid_set)
            os.system(pid_cmd)


def cleanup():
    """
    cleanup device wda process
    :return:
    """
    pid_list = []
    cmd = decryption(b'dGlkZXZpY2U=')
    sub = subprocess.Popen(f'{cmd} ps', shell=True, close_fds=True, stdout=subprocess.PIPE)
    sub.wait()
    pid_info = sub.stdout.read().decode().splitlines()
    for u in pid_info:
        if 'WebDriverAgentRunner' in u:
            pid_list.append(u.strip().split(' ')[0])
    [os.system(f'{cmd} kill {pid}') for pid in pid_list]


def display():
    """
    Gets the length and width of the current device
    :return:
    """
    width, height = device().get_current_resolution()
    return width, height


def device_udid(state, types: str):
    """
    Perform `adb devices` command and return the list of adb devices
    Perform `utx list` command and return the list of iphone devices
    :param types: mobile platform
    :param state: optional parameter to filter devices in specific state
    :return: list od android devices or ios devices
    """
    device_list = []
    if types.lower() == 'android':
        patten = re.compile(r'^[\w\d.:-]+\t[\w]+$')
        output = ADB().cmd("devices", device=False)
        for line in output.splitlines():
            line = line.strip()
            if not line or not patten.match(line):
                continue
            serialno, cstate = line.split('\t')
            if state and cstate != state:
                continue
            device_list.append(serialno)
    elif types.lower() == 'ios':
        # Get the udid list of the connected mobile phone
        sub = subprocess.Popen('utx list', shell=True, close_fds=True, stdout=subprocess.PIPE)
        sub.wait()
        udid = sub.stdout.read().decode().splitlines()
        for u in udid:
            us = u.strip().split(' ')[0]
            if us != 'UDID':
                device_list.append(us)
    return device_list


def ios_device_info():
    """
    Gets device_info of the current device

    :return:
    """
    res = subprocess.run('{} info'.format(decryption(b'dGlkZXZpY2U=')), shell=True, stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    lines = res.stdout.decode("utf-8", "ignore")
    device_info = [info for info in lines.split('\n') if info]
    _device = {}
    if len(device_info) < 2:
        LOGGING.error(f'Read device info line error. {lines}')
    for info in device_info:
        info_kv = info.split(':')
        if info_kv[0] == 'ProductVersion':
            _device['ProductVersion'] = info_kv[1].strip()
        if info_kv[0] == 'MarketName':
            _device['MarketName'] = info_kv[1].strip()
        if info_kv[0] == 'SerialNumber':
            _device['SerialNumber'] = info_kv[1].strip()
    return _device


def get_report(airtest_report_path):
    """
    Get the latest test report path
    :return: report name and path
    """
    file_lists = os.listdir(airtest_report_path)
    file_lists.sort(key=lambda fn: os.path.getmtime(airtest_report_path + "/" + fn)
    if not os.path.isdir(airtest_report_path + "/" + fn) else 0)
    report = os.path.join(airtest_report_path, file_lists[-1])
    print(file_lists[-1])
    return report


def encryption(value):
    """
    encryption
    https://cdn.jsdelivr.net/gh/openutx/static/
    :param value:
    :return:
    """
    bytes_url = value.encode("utf-8")
    str_url = base64.b64encode(bytes_url)
    return str_url


def decryption(value):
    """
    decryption
    https://cdn.jsdelivr.net/gh/openutx/static/
    :param value:
    :return:
    """
    str_url = base64.b64decode(value).decode("utf-8")
    return str_url


def str_to_bool(value):
    """
    str convert bool
    :param value:
    :return:
    """
    return True if value.lower() == 'true' else False


def find_all_cases(base, status, mark):
    """

    :param base:
    :param status:
    :param mark:
    :return:
    """
    for root, ds, fs in os.walk(base, topdown=True):
        if status in ["0", False]:
            for f in ds:
                if f.endswith('.air') and mark in f:
                    fullname = os.path.join(root, f)
                    yield fullname
        else:
            for f in ds:
                if f.endswith('.air'):
                    fullname = os.path.join(root, f)
                    yield fullname


def get_host_ip():
    """
    Query the local ip address
    :return:
    """
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        s.connect(('8.8.8.8', 80))
        ip = s.getsockname()[0]
    finally:
        s.close()

    return ip


def proxy(devices, status=True):
    """
    android proxy
    :param devices:
    :param status: on or off
    :return:
    :platforms: Android
    """
    if status:
        proxy_cmd = f"{ADB.builtin_adb_path()} -s {devices} {decryption(b'c2hlbGwgc2V0dGluZ3MgcHV0IGdsb2JhbCBodHRwX3Byb3h5')} {get_host_ip()}:8888"
        logger.info(f'Successfully enabled {devices} global proxy！')
        os.system(proxy_cmd)
    else:
        proxy_cmd = f"{ADB.builtin_adb_path()} -s {devices} {decryption(b'c2hlbGwgc2V0dGluZ3MgcHV0IGdsb2JhbCBodHRwX3Byb3h5IDow')}"
        logger.info(f'Close the {devices} global proxy successfully！')
        os.system(proxy_cmd)


def selector(status, flag, cases_list):
    """

    :param status:
    :param flag:
    :param cases_list:
    :return:
    """
    cases = []
    if status in ["0", False]:
        for suite in cases_list:
            if flag in suite:
                if suite.endswith(".air"):
                    cases.append(suite)
    else:
        for suite in cases_list:
            if suite.endswith(".air"):
                cases.append(suite)

    return cases


def selector_v2(path, status, mark):
    cases = []
    for i in find_all_cases(path, status, mark):
        print(i)
        cases.append(i)
    return sorted(cases)


def selector_v3(path: str, status, mark):
    """

    :param path:
    :param status:
    :param mark:
    :return:
    """
    cases = []
    base_path = path.split(path.split('/')[-1])[0]
    path_list = ''.join(str(path).split()[0].split('/')[-1]).split(',')
    for item in path_list:
        for i in find_all_cases(base_path + item, status, mark):
            print(i)
            cases.append(i)
    return sorted(cases)


def download(file_url, types):
    """

    :param types:
    :param file_url:
    :return:
    """
    if 'http' not in file_url:
        return
    download_url = 'http' + file_url.split('http')[-1]
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    r = requests.get(download_url, stream=True, verify=False)
    total = int(r.headers.get('content-length', 0))
    filename = "{}utx_download_{}.{}".format(file_url.split('http')[0], int(round(time.time() * 1000)), types)

    with open(filename, 'wb') as file, tqdm(
            desc=filename,
            total=total,
            unit='iB',
            unit_scale=True,
            unit_divisor=1024,
    ) as bar:
        for data in r.iter_content(chunk_size=1024):
            size = file.write(data)
            bar.update(size)

    return filename


def start_recording(max_time=1800):
    """

    :param max_time:
    :return:
    """
    serialno = device().serialno
    model = ADB(serialno=serialno)
    recorder = Recorder(model)
    recorder.start_recording(max_time=max_time)


def stop_recording(file, name=None):
    """

    :param file: __file__
    :param name: mp4 name
    :return: video file
    """
    serialno = device().serialno
    model = ADB(serialno=serialno)
    build_version = shell('getprop ro.build.version.release')
    dev = model.get_model() + ' Android {}'.format(build_version).replace('\n', '').replace('\r', '')
    recorder = Recorder(model)
    if not name:
        name = str(file).split('/')[-1].split('.')[0]
    output_path = os.path.join(str(file).split('.air')[0] + '.air', 'log', dev, name)
    recorder.stop_recording(output=f'{output_path}.mp4')


def check_network():
    """
    :return:
    """
    try:
        result = shell('ping -c 1 www.baidu.com')
        logger.info(result)
        return True
    except AdbShellError as e:
        logger.error(e)
        return False


def proxy_dump():
    """
    proxy filter url address
    :return:
    """

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"Created file: {path}"
        print(msg)

    content = """#!/usr/bin/python
# encoding=utf-8

\"\"\" Can only be modified by the administrator. Only proxy are provided.
\"\"\"

from garbevents.capture import GetData

addons = [
    GetData()
]
"""
    create_file(os.path.join("proxy.py"), content)
    try:
        from garbevents.cli.main import mitmweb
        from garbevents.settings import Settings as ST
        mitmweb(args=["-p", f"{ST.server_port}", "--ssl-insecure", "--web-host", f"{ST.web_host}", "--web-port",
                      f"{ST.web_port}", "--no-web-open-browser", "-s", "proxy.py"])
    except (ImportError, AttributeError):
        logger.error(
            "garbevents not installed, please run: pip install -U garbevents")
        return


def launcher(main, salve):
    """

    :param main:
    :param salve:
    :return:
    """
    master = Process(target=main)
    salver = Process(target=salve)
    salver.daemon = True
    master.daemon = True
    salver.start()
    master.start()
    master.join()


class ReadConfig:
    """
    configuration file
    """

    def __init__(self, ini_path):
        self.ini_path = ini_path
        if not os.path.exists(ini_path):
            raise FileNotFoundError("Profile %s does not exist！" % ini_path)
        self.config = configparser.RawConfigParser()  # When there are% symbols, use raw to read
        self.config.read(ini_path, encoding='utf-8')

    def _get(self, section, option):
        """

        :param section:
        :param option:
        :return:
        """
        return self.config.get(section, option)

    def _set(self, section, option, value):
        """

        :param section:
        :param option:
        :param value:
        :return:
        """
        try:
            self.config.set(section, option, value)
        except configparser.NoSectionError as e:
            logger.error(e)
        with open(self.ini_path, 'w') as f:
            self.config.write(f)

    def getvalue(self, env, name):
        return self._get(env, name)

    def update_value(self, env, name, value):
        return self._set(env, name, value)
